@@ -19,10 +19,12 @@
 # 02.03.2024: Add $boot_device
 # 17.03.2024: New backup host/share handling.
 # 17.03.2024: Add DB backup script for nextcloud
+# 30.12.2024: Fix design error to hanfle snapshots more safty 
 
 ### configuration ###
 
 date="$(/usr/bin/date "+%F")"
+failed_ret=0
 
 if [ -f /usr/local/etc/make_backup.conf ]; then
    source /usr/local/etc/make_backup.conf || { echo "Can't read configuration file!"; exit 255; }
@@ -31,13 +33,21 @@ else
    exit 255
 fi
 
+# shellcheck disable=SC2154
 if [ "$new_backup" == "yes" ]; then
    backupfile="/backup/system-${date}.tar.gz"
 else
+   # shellcheck disable=SC2154
    backupfile="/backup/${hostname}/system-${date}.tar.gz"
 fi
 working_dir="/usr/local/sbin"
 
+### check conditions
+if [ -f /.backup_run ]; then
+   echo "Lockfile from previous backup found! Aborting..."
+   exit 255
+fi
+
 ### switch to working dir to use relative path
 cd "$working_dir" || { echo "Can't switch to working dir"; exit 255; }
 
@@ -51,6 +61,9 @@ fi
 
 ### create and mount snapshots under /mnt ###
 
+# create lock file
+touch /.backup_run || exit 255
+
 for i in lvm_pre_*; do
    if [ "$1" == "boot" ]; then
       if [ "$i" == "lvm_pre_0_snap_make.sh" ]; then
@@ -60,14 +73,30 @@ for i in lvm_pre_*; do
    ret=$(exec $i) 
    if [ "$ret" == "255" ]; then
       echo "Fatal error on $i"
-      exit 255
+      failed_ret=255 
    fi
 done
 
+# error handling
+if [ $failed_ret == 255 ]; then
+   ret=$(exec lvm_post_1_delete.sh)
+   if [ "$ret" == 255 ]; then
+      # TODO: send mail
+      while true; do
+         echo -e '\a' > /dev/console
+         sleep 5
+         if [ -f /.backup_exit ]; then
+            exit 255
+         fi
+      done 
+   fi
+fi   
+
 ### Unmount and mount /boot under /mnt/boot ###
 
-/usr/bin/umount /boot || { echo "Can't unmount /boot"; exit 255; }
-/usr/bin/mount -o ro $boot_device /mnt/boot || { echo "Can't mount boot under /mnt/boot"; exit 255; }
+/usr/bin/umount /boot || { echo "Can't unmount /boot"; failed_ret=255; }
+# shellcheck disable=SC2154
+/usr/bin/mount -o ro "$boot_device" /mnt/boot || { echo "Can't mount boot under /mnt/boot"; failed_ret=255; }
 
 ### Do backup ###
 
@@ -77,18 +106,22 @@ else
    output="-v"
 fi
 
-echo "Starting backup..."
+if  [ $failed_ret != 255 ]; then
+   echo "Starting backup..."
 
-if [ "$use_pigz" == "yes" ]; then
-	/usr/bin/tar --xattrs -cp  $output -C /mnt . |pigz -p $use_pigz_threads > "$backupfile"
-else
-	/usr/bin/tar --xattrs -czp $output -f "$backupfile" -C /mnt .
+   # shellcheck disable=SC2154
+   if [ "$use_pigz" == "yes" ]; then
+      # shellcheck disable=SC2154
+      /usr/bin/tar --xattrs -cp  $output -C /mnt . |pigz -p "$use_pigz_threads" > "$backupfile" || failed_ret=$?
+   else
+      /usr/bin/tar --xattrs -czp $output -f "$backupfile" -C /mnt . || failed_ret=$?
+   fi
 fi
 
 ### Unmount and mount boot back to /boot ###
 
-/usr/bin/umount /mnt/boot || { echo "Can't unmount /backup/mnt/boot"; exit 255; }
-/usr/bin/mount /boot || { echo "Can't mount boot back to /boot"; exit 255; } 
+/usr/bin/umount /mnt/boot || { echo "Can't unmount /backup/mnt/boot"; failed_ret=255; }
+/usr/bin/mount /boot || { echo "Can't mount boot back to /boot"; failed_ret=255; } 
 
 ### cleanup ###
 
@@ -96,10 +129,25 @@ for i in lvm_post_*; do
    ret=$(exec $i) 
    if [ "$ret" == "255" ]; then
       echo "Fatal error on $i"
-      exit 255
+      failed_ret=255
    fi
 done
 
+# error handling
+if [ $failed_ret == 255 ]; then
+   ret=$(exec lvm_post_1_delete.sh)
+   if [ "$ret" == 255 ]; then
+      # TODO: send mail
+      while true; do
+         echo -e '\a' > /dev/console
+         sleep 5
+         if [ -f /.backup_exit ]; then
+            exit 255
+         fi
+      done
+   fi
+fi
+
 ### Copy backup ###
 
 if [ "$hostname" == "daten-box" ]; then
@@ -111,7 +159,10 @@ if [ "$hostname" == "nextcloud" ]; then
    /usr/local/sbin/make_db_dump.sh /backup
 fi
 
-/usr/bin/umount /backup
+# clean up work
+/usr/bin/umount /backup || exit 255
+rm /.backup_run || exit 255
 
 echo "Everything runs smoothly :-)"
-exit 0 
+
+exit $failed_ret 
